"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BaseManager = void 0;

var _apolloServer = require("apollo-server");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var _authorized = {
  get: {
    groups: ["admin"]
  },
  query: {
    groups: ["admin"]
  },
  create: {
    groups: ["admin"]
  },
  update: {
    groups: ["admin"]
  },
  "delete": {
    groups: ["admin"]
  }
};

var BaseManager =
/*#__PURE__*/
function () {
  function BaseManager(modelName, type, container) {
    _classCallCheck(this, BaseManager);

    this.modelName = modelName;
    this.type = type;
    this.container = container;
    this.log = container.log;
    this.session = container.session;
  }

  _createClass(BaseManager, [{
    key: "authorized",
    value: function authorized() {
      return _authorized;
    }
  }, {
    key: "getNewInstance",
    value: function getNewInstance() {
      return new this.type();
    } // ===================================================
    // perform authentication and authorization

  }, {
    key: "authorize",
    value: function () {
      var _authorize = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(action, doAuthorize, obj, auth) {
        var userInfo, actionAuth, checkOwner, isOwner, authUserId, isInGroups;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (doAuthorize) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt("return");

              case 2:
                _context.next = 4;
                return this.session.get(auth.token);

              case 4:
                userInfo = _context.sent;

                if (userInfo) {
                  _context.next = 7;
                  break;
                }

                throw new _apolloServer.AuthenticationError("User session is not authenticated");

              case 7:
                actionAuth = this.authorized()[action] || {};
                checkOwner = actionAuth.owner || false;
                isOwner = true;

                if (checkOwner) {
                  authUserId = userInfo.user._id || "";
                  isOwner = obj.isOwner(authUserId);
                }

                isInGroups = this.userInGroups(actionAuth.groups, userInfo);

                if (isInGroups || checkOwner && isOwner) {
                  _context.next = 14;
                  break;
                }

                throw new _apolloServer.ForbiddenError("User is not authorized ");

              case 14:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function authorize(_x, _x2, _x3, _x4) {
        return _authorize.apply(this, arguments);
      }

      return authorize;
    }()
  }, {
    key: "applyReadPermissions",
    value: function () {
      var _applyReadPermissions = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2(doAuthorize, obj, auth) {
        var userInfo, authUserId, secureGroups, typeName, isInSecureGroups;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (doAuthorize) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt("return", obj);

              case 2:
                _context2.next = 4;
                return this.session.get(auth.token);

              case 4:
                userInfo = _context2.sent;

                if (userInfo) {
                  _context2.next = 7;
                  break;
                }

                throw new _apolloServer.AuthenticationError("User session is not authenticated");

              case 7:
                authUserId = userInfo.user._id || "";
                secureGroups = obj.getPermissions("Secure");
                typeName = obj.constructor.name;
                isInSecureGroups = this.userInGroups(secureGroups, userInfo);

                if (!isInSecureGroups) {
                  _context2.next = 15;
                  break;
                }

                return _context2.abrupt("return", obj);

              case 15:
                return _context2.abrupt("return", obj.applyReadPermissions(authUserId));

              case 16:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function applyReadPermissions(_x5, _x6, _x7) {
        return _applyReadPermissions.apply(this, arguments);
      }

      return applyReadPermissions;
    }()
  }, {
    key: "applyQueryReadPermissions",
    value: function () {
      var _applyQueryReadPermissions = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3(doAuthorize, items, auth) {
        var userInfo, authUserId, obj, secureGroups, isInSecureGroups, filtered;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (doAuthorize) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt("return", items);

              case 2:
                _context3.next = 4;
                return this.session.get(auth.token);

              case 4:
                userInfo = _context3.sent;

                if (userInfo) {
                  _context3.next = 7;
                  break;
                }

                throw new _apolloServer.AuthenticationError("User session is not authenticated");

              case 7:
                authUserId = userInfo.user._id || "";
                obj = this.getNewInstance();
                secureGroups = obj.getPermissions("Secure");
                isInSecureGroups = this.userInGroups(secureGroups, userInfo);

                if (isInSecureGroups) {
                  _context3.next = 16;
                  break;
                }

                filtered = items.map(function (obj) {
                  return obj.applyReadPermissions(authUserId);
                });
                return _context3.abrupt("return", filtered);

              case 16:
                return _context3.abrupt("return", items);

              case 17:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function applyQueryReadPermissions(_x8, _x9, _x10) {
        return _applyQueryReadPermissions.apply(this, arguments);
      }

      return applyQueryReadPermissions;
    }()
    /**
     * Allow for future softening of user->group membership system
     */

  }, {
    key: "userInGroups",
    value: function userInGroups(groupList, userInfo) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = groupList[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var group = _step.value;

          switch (group) {
            case "*":
              return true;
              break;

            case "admin":
              if (userInfo.user.administrator) {
                return true;
                break;
              }

          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return false;
    }
  }]);

  return BaseManager;
}();

exports.BaseManager = BaseManager;
//# sourceMappingURL=baseManager.js.map