"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BaseType = exports.Permissions = void 0;

var _suryaGqlScalar = require("surya-gql-scalar");

var _util = require("util");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Permissions = {
  Public: "Public",
  // can be accessed by any user
  Secure: "Secure",
  // can be accessed by a user with Secure privilege on object type
  Personal: "Personal",
  // can be accessed by a user who owns the immediate object
  Internal: "Internal" // is not exposed on external api, but is on the API returned object

};
exports.Permissions = Permissions;

var BaseType =
/*#__PURE__*/
function () {
  function BaseType(data) {
    _classCallCheck(this, BaseType);

    if (data) {
      this.loadData(data);
    }
  }
  /**
   * Override typeSpec() to return definition of type global properties and fields.
   */


  _createClass(BaseType, [{
    key: "typeSpec",
    value: function typeSpec() {
      return {
        name: "BaseType",
        "extends": null,
        instantiable: false,
        timestamps: false,
        permissions: {
          Secure: ["auth"]
        },
        fields: {}
      };
    }
  }, {
    key: "getOwnerField",
    value: function getOwnerField() {
      var spec = this.typeSpec();
      var ownerFieldName = null;

      if (spec && spec.fields && (0, _util.isObject)(spec.fields)) {
        for (var _i = 0, _Object$keys = Object.keys(spec.fields); _i < _Object$keys.length; _i++) {
          var fieldName = _Object$keys[_i];
          var fieldSpec = spec.fields[fieldName];

          if (fieldSpec.ownerId) {
            ownerFieldName = fieldName;
            break;
          }
        }
      }

      return ownerFieldName;
    }
  }, {
    key: "getKeyField",
    value: function getKeyField() {
      var spec = this.typeSpec();
      var keyFieldName = null;

      if (spec && spec.fields && (0, _util.isObject)(spec.fields)) {
        for (var _i2 = 0, _Object$keys2 = Object.keys(spec.fields); _i2 < _Object$keys2.length; _i2++) {
          var fieldName = _Object$keys2[_i2];
          var fieldSpec = spec.fields[fieldName];

          if (fieldSpec.key) {
            keyFieldName = fieldName;
            break;
          }
        }
      }

      return keyFieldName;
    }
  }, {
    key: "getOwnerId",
    value: function getOwnerId() {
      var ownerFieldName = this.getOwnerField();
      return this[ownerFieldName];
    }
    /**
     * Default implementation of isOwner logic.
     * Override this logic in cases where the userId of the owner is not directly stored as a data field of the object
     *
     * @param {*} userId
     */

  }, {
    key: "isOwner",
    value: function isOwner(userId) {
      return this.getOwnerId() == userId;
    }
  }, {
    key: "getPermissions",
    value: function getPermissions(permType) {
      var typeSpec = this.typeSpec();

      if (typeSpec.permissions) {
        return typeSpec.permissions[permType];
      }

      return [];
    }
    /**
     * Apply permission filters
     * Expect to be called only if user is not Secure (has permissions to Secure fields.)
     * filter out secure fields
     * filter out Personal fields unless the user is the owner
     * @param {*} authUserId
     */

  }, {
    key: "applyReadPermissions",
    value: function applyReadPermissions(authUserId) {
      var _this = this;

      var isOwner = this.isOwner(authUserId);
      var filtered = Object.create(this);
      filtered.loadData(this);
      var className = this.constructor.name;
      var spec = this.typeSpec();

      if (spec && spec.fields && (0, _util.isObject)(spec.fields)) {
        Object.keys(spec.fields).forEach(function (fieldName) {
          var fieldSpec = spec.fields[fieldName];

          _this._filterDataField(isOwner, fieldName, fieldSpec, filtered);
        });
      }

      return filtered;
    }
  }, {
    key: "_filterDataField",
    value: function _filterDataField(isOwner, fieldName, fieldSpec, filtered) {
      var readPermission = fieldSpec.readPermission || Permissions.Public;
      var filter = readPermission == Permissions.Secure || readPermission == Permissions.Personal && !isOwner;
      if (filter) filtered[fieldName] = null;
    }
    /**
     * Load basic JSON data per spec
     * @param {*} data
     */

  }, {
    key: "loadData",
    value: function loadData(data) {
      var _this2 = this;

      if (data) {
        var spec = this.typeSpec();

        if (spec && spec.fields && (0, _util.isObject)(spec.fields)) {
          Object.keys(spec.fields).forEach(function (fieldName) {
            var fieldSpec = spec.fields[fieldName];

            _this2._loadDataField(fieldName, fieldSpec, data);
          });
        }
      }

      return this;
    }
  }, {
    key: "_loadDataField",
    value: function _loadDataField(fieldName, fieldSpec, data) {
      var useFieldName = fieldSpec.type == _suryaGqlScalar.ScalarTypes.ObjectRef ? fieldSpec.refField : fieldName;
      this[useFieldName] = data[useFieldName];
    }
    /**
     * Load data from mongo document with necessary modifications
     * TODO: Move this to and adapter model hosted in the API
     * @param {*} data
     */

  }, {
    key: "loadDocument",
    value: function loadDocument(data) {
      var _this3 = this;

      var spec = this.typeSpec();

      if (spec && spec.fields && (0, _util.isObject)(spec.fields)) {
        Object.keys(spec.fields).forEach(function (fieldName) {
          var fieldSpec = spec.fields[fieldName];

          _this3._loadDocField(fieldName, fieldSpec, data);
        });
      }

      return this;
    }
  }, {
    key: "_loadDocField",
    value: function _loadDocField(fieldName, fieldSpec, doc) {
      if (!doc) return;
      var data = doc._doc; // Otherwise non-string types are not getting.

      var useFieldName = fieldName;

      if (!!fieldSpec.key) {
        this[fieldName] = data._id.toString();
      } else {
        if (fieldSpec.type == _suryaGqlScalar.ScalarTypes.ObjectRef) {
          useFieldName = fieldSpec.refField;
        }

        var value = data[useFieldName];

        switch (fieldSpec.type) {
          case _suryaGqlScalar.ScalarTypes.DateTime:
            // do nothing. we expect a Date object in the Date type
            break;
        }

        this[useFieldName] = value;
      }
    }
  }, {
    key: "toGraphQLSchema",
    value: function toGraphQLSchema() {
      var spec = this.typeSpec();
      this.schemaTypeName = spec.name;

      var typeSchema = this._genType(spec);

      var inputSchema = this._genInput(spec);

      var setResponseSchema = this._genSetResponseType(spec);

      var mutationResponseSchema = this._genMutationResponseType(spec);

      var specInputSchemas = this._genSpecInputs(spec);

      var inlineScalars = this._genScalars(spec); // console.log(`Scalars [${this.schemaTypeName}]: \n> ${inlineScalars};`)


      var schemas = typeSchema + "\n" + inputSchema + "\n" + setResponseSchema + "\n" + mutationResponseSchema + "\n" + specInputSchemas + "\n" + inlineScalars + "\n";
      return schemas;
    }
  }, {
    key: "_genType",
    value: function _genType(spec) {
      return "type ".concat(spec.name, " {\n").concat(this._genFields(spec.fields, false, null), "\n}");
    }
  }, {
    key: "_genInput",
    value: function _genInput(spec) {
      return "input ".concat(spec.name, "Input {\n").concat(this._genFields(spec.fields, true, null), "\n}");
    }
  }, {
    key: "_genSpecInputs",
    value: function _genSpecInputs(spec) {
      var inputNames = spec.inputs || [];
      var specInputs = [];

      for (var n = 0; n < inputNames.length; n++) {
        var inputName = inputNames[n];
        var specInput = "input ".concat(inputName, " {\n        ").concat(this._genFields(spec.fields, true, inputName), "\n        }");
        specInputs.push(specInput);
      }

      return specInputs.join("/n");
    }
  }, {
    key: "_genSetResponseType",
    value: function _genSetResponseType(spec) {
      if (!!spec.instantiable) {
        return "type ".concat(spec.name, "SetResponse {\n  items: [").concat(spec.name, "]!\n  total: Int\n  count: Int\n  offset: Int\n}");
      } else {
        return "";
      }
    }
  }, {
    key: "_genMutationResponseType",
    value: function _genMutationResponseType(spec) {
      if (!!spec.instantiable) {
        return "type ".concat(spec.name, "MutationResponse {\n  success: Boolean!\n  message: String\n  data: ").concat(spec.name, "\n}\n");
      } else {
        return "";
      }
    }
  }, {
    key: "_genFields",
    value: function _genFields(fields) {
      var _this4 = this;

      var input = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var specInput = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      var fstr = Object.getOwnPropertyNames(fields).map(function (name) {
        var typedef = fields[name];

        var _this4$_typeMap = _this4._typeMap(name, typedef, input, specInput),
            fieldName = _this4$_typeMap.fieldName,
            fieldType = _this4$_typeMap.fieldType,
            include = _this4$_typeMap.include,
            params = _this4$_typeMap.params,
            scalar = _this4$_typeMap.scalar;

        if (include) {
          return "    ".concat(fieldName).concat(params, ": ").concat(fieldType, "\n");
        } else {
          return "";
        }
      }).join(" ");
      return fstr;
    }
  }, {
    key: "_genScalars",
    value: function _genScalars(spec) {
      var _this5 = this;

      var fields = spec.fields;
      var fstr = Object.getOwnPropertyNames(fields).map(function (name) {
        var typedef = fields[name];

        var _this5$_typeMap = _this5._typeMap(name, typedef, false, ""),
            fieldName = _this5$_typeMap.fieldName,
            fieldType = _this5$_typeMap.fieldType,
            include = _this5$_typeMap.include,
            params = _this5$_typeMap.params,
            scalar = _this5$_typeMap.scalar;

        if (scalar.length > 0) {
          return "".concat(scalar, "\n");
        }
      }).join(" ");
      return fstr;
    }
  }, {
    key: "_typeMap",
    value: function _typeMap(fieldName, typeDef) {
      var input = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var specInput = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
      var typeName = typeDef.type;
      var required = !!typeDef.required;
      var permission = typeDef.readPermission || Permissions.Public;
      var include = (!input || input && typeDef.input !== false) && permission != Permissions.Internal;
      var pre = "";
      var post = "";
      var params = "";

      if (typeDef.array) {
        pre = "[";
        post = "]";
      }

      var bang = required ? "!" : "";
      var useFieldName = fieldName;
      var scalar = ""; // auxiliar scalars such as enums

      var gqlType = "String";

      switch (_typeName) {
        case _suryaGqlScalar.ScalarTypes.String:
          gqlType = pre + "String" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.DateTime:
          gqlType = pre + "Date" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.Integer:
          gqlType = pre + "Int" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.Float:
          gqlType = pre + "Float" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.ID:
          gqlType = pre + "ID" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.Boolean:
          gqlType = pre + "Boolean" + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.Object:
          var _typeName = typeDef.objectType + (input ? "Input" : "");

          gqlType = pre + _typeName + bang + post;
          break;

        case _suryaGqlScalar.ScalarTypes.ObjectRef:
          var paramType = typeDef.params || "";

          if (input) {
            var specInputs = typeDef.inputs || [];

            if (specInput && specInputs.includes(specInput)) {
              // embed as if it were an input object
              var _typeName2 = typeDef.objectType + (input ? "Input" : "");

              gqlType = pre + _typeName2 + bang + post;
            } else {
              // A usual input id reference
              if (typeDef.refField) {
                useFieldName = typeDef.refField;
                gqlType = pre + typeDef.refFieldType + bang + post;
              } else {
                // if no refField specified, we assume it's not insertable
                include = false;
              }
            }
          } else {
            gqlType = pre + typeDef.objectType + bang + post;

            if (paramType == "query") {
              params = "(offset: Int, limit: Int, filter: String, sort: String)";
            }
          }

          break;

        case _suryaGqlScalar.ScalarTypes.Enum:
          var enumName = "".concat(this.schemaTypeName, "_").concat(useFieldName, "_ENUM").toUpperCase();
          var scalarEnum = typeDef["enum"].join("\n");
          scalar = "enum ".concat(enumName, " {\n").concat(scalarEnum, "\n}");
          gqlType = pre + enumName + bang + post;
          break;
      }

      return {
        fieldName: useFieldName,
        fieldType: gqlType,
        include: include,
        params: params,
        scalar: scalar
      };
    }
  }]);

  return BaseType;
}();

exports.BaseType = BaseType;
//# sourceMappingURL=baseType.js.map